; ZXodus Engine
; Copyright (c) 2011 Andrew Owen

report_j	equ	0x15c4
chan_open	equ	0x1601
make_room	equ	0x1655
pixel_add	equ	0x22aa
strms		equ	0x5c10
chans		equ	0x5c4f
prog		equ	0x5c53
coords		equ	0x5c7d
coords_x	equ	0x5c7d
coords_y	equ	0x5c7e
df_cc		equ	0x5c84
s_posn		equ	0x5c88
attr_p		equ	0x5c8d

org 39748
on:
	im		2
	ret

org 39751
off:
	im		1
	ret

org 39754
print6x8:					; 6x8 print code (321 bytes)
	push	hl				; Save all the	registers, call the 
	push	bc				; printing routine, put all the registers
	push	de				; back again and leap back to the
	push	af				; operating system.
	call	doit1
	pop		af
	pop		de
	pop		bc
	pop		hl
	ret

doit1:
	push	af				; Look to see if last character was a
	ld		a, (atflg)		; control code 22 - the code for AT.
	and		a
	jr		nz, getxp
	pop		af

atchq:
	cp		22				; If the current character is an AT control
	jr		nz, crchq		; code, set the ATFLG to indicate that the
	ld		a, 255			; next two codes dealt with will be the X
	ld		(atflg), a		; and Y positions for the print.
	ret

getxp:
	cp		254				; If the last charater was an AT then fetch
	jr		z, getyp		; the X and Y co-ordinates and move to the
	pop		af				; new printing position.
	ld		(coords_x), a
	ld		hl, atflg
	dec		(hl)
	ret

getyp:
	pop		af
	ld		b, a
	ld		a, 184
	sub		b
	ld		(coords_y), a
	xor		a
	ld		(atflg), a
	ret

crchq:
	cp		13				; If the current character is a return 
	jr		nz, vchrq		; control code, move down 8 pixels and open
	call	dwncr			; channel 2 to deal with nasty INK and
	ld		a, 2			; PAPER control codes.
	call	chan_open

skipc:
	ret	

vchrq:
	cp		24				; Burp! if the character is not between 32
	jr		c, prntq		; and 127 then print a question mark
	cp		128				; instead.
	jr		c, fchr

prntq:
	ld		a, '?'
	jr		fchr
	
get_addr:
	ld		h, 0			; Figure out where the character's
	ld		l, a			; definition is stored in memory.
	add		hl, hl
	add		hl, hl
	add		hl, hl
	ex		de, hl
	ret

err5:
	rst		8
	defb	4

fchr:
	call	get_addr
	ld		hl, font - 256

udg2:
	add		hl, de

prnit:
	ld		bc, 7			; the address of the seventh byte of the
	add		hl, bc			; character, check the character will fit
	ld		(chrad), hl		; on the screen and calculate the address
	ld		a, (coords_y)	; in the display file where the character
	cp		185				; will be printed.
	jr		nc, err5
	call	fitcq
	ld		bc, (coords)
	ld		a, 191			; mod to access lower screen
	call	pixel_add + 2
	ld		(s_posn), a
	ld		(df_cc), hl
	ld		b, 8

prnlp:
	push	bc
	ld		hl, (chrad)
	ld		a, (hl)
	sll		a				; pre-rotate left once to get the middle
	and		%11111100		; six and clear the right two pixels
	dec		hl
	ld		(chrad), hl
	ld		l, a
	ld		a, (s_posn)
	ld		bc, 0xff03		; mask
	push	bc				; save mask
	and		a				; sometimes no rotation is required
	jr		z, putit
	pop		bc				; drop saved mask
	ld		b, a
	push	bc				; save count	
	ex		de, hl			; save character

mask:
	ld		hl, 0x00fc

rotmsk:
	srl		l
	rr		h
	and		a
	djnz	rotmsk
	ld		a, 255			; invert mask
	xor		h
	ld		h, a
	ld		a, 255
	xor		l
	ld		l, a
	pop		bc				; restore count
	push	hl				; save mask
	ex		de, hl			; restore character
	ld		h, 0

rotlp:
	srl		l				; Rotate the character definition into the 
	rr		h				; correct pixel position, place it on the
	and		a				; screen one byte at a time and make sure
	djnz	rotlp			; each byte is in the right colour.

putit:
	pop		bc				; restore mask
	ld		de, (df_cc)
	ld		a, (de)
	and		c				; mask off
	xor		l
	ld		(de), a
	call	colad
	ld		a, (s_posn)
	and		a
	jr		z, pst
	inc		e
	ld		a, e
	and		31
	jr		z, pst
	ld		a, (coords_x)
	or		248
	add		a, 4			; always 6 pixels 
	jr		nc, pst
	ld		a, (de)
	and		b				; mask off
	xor		h
	ld		(de), a
	call	colad

pst:
	ld		hl, (df_cc)
	call	uline
	ld		(df_cc), hl
	pop		bc
	djnz	prnlp
	ld		a, (coords_x)	; the character just printed.
	add		a, 6			; always 6 pixels
	ld		(coords_x), a
	ret

uline:
	push	af				; Here's a handy routine that sets HL to
	ld		a, h			; point to the next pixel line up in the
	dec		h				; display file.
	and		7
	jr		nz, _end
	ld		a, l
	sub		32
	ld		l, a
	jr		c, _end
	ld		a, h
	add		a, 8
	ld		h, a

_end:
	pop		af
	ret

fitcq:
	ld		b, 8			; Check if the character to be printed will fit
	ld		a, %00000011	; Always 6 pixels
	ld		c, a

cntlp:
	and		a
	srl		c  
	jr		nc, _out
	dec		b
	jr		cntlp

_out:
	ld		a, b
	ld		a, (coords_x)
	add		a, b
	ret		nc

dwncr:
	xor		a				; If it doesn't, move down 8 pixels and
	ld		(coords_x), a	; back to the left hand side of the screen.
	ld		a, (coords_y)
	sub		8
	ld		(coords_y), a
	ret

colad:
	push	hl				; And another useful routine - it
	push	af				; calculates the relevant address in the
	ld		a, d			; attributes file from a  given display
	rrca					; file address and stores the INK value of
	rrca					; ATTR_P (the permanent PAPER and INK
	rrca					; colours) in it.
	and		3
	or		%01011000
	ld		h, a
	ld		l, e
	ld		a, (attr_p)
	ld		(hl), a
	pop		af
	pop		hl
	ret						; That's all folks!


org 40075
tile_map:
REPT 81, tileno
	defb	tileno			; sets the tiles that will be drawn (81 bytes)
ENDM

org 40075
initialize:					; this code need only be run once
	ld		a, 59
	ld		i, a

c_chan:
	ld		hl, (prog)		; a channel must be created below basic
							; so look at the system variable PROG
	dec		hl				; move hl down one address
	ld		bc, 5			; the new channel takes 5 bytes
	call	make_room		; call the MAKE_ROOM routine
	inc		hl				; move HL up one address
	ld		bc, print6x8	; could write the bytes directly but
							; then code would be non-relocatable
	ld		(hl), c			; low byte of the output routine
	inc		hl				; move HL up one address
	push	hl				; save this address for later
	ld		(hl), b			; high byte of the output routine
	inc		hl				; move HL up one address
	ld		bc, report_j	; address of input routine (REPORT J)
	ld		(hl), c			; low byte of the input routine
	inc		hl				; move HL up one address
	ld		(hl), b			; high byte of the input routine
	inc		hl				; move HL up one address 
	ld		(hl), 'S'		; channel type; 'K', 'S', 'R' or 'P'

; attach stream

	pop		hl				; the first address plus one of the
							; extra space stored earlier
	ld		de, (chans)		; store the contents of CHANS in DE
	and		a				; clear the carry flag before
							; calculation
	sbc		hl, de			; the difference between the start of
							; the channels area and the start of the
							; extra space becomes the offset, stored
							; in HL
	ex		de, hl			; store the offset in DE
	ld		hl, strms		; store the contents of STRMS in HL
	ld		a, 4			; stream number 4
	add		a, 3			; take account of streams -3 to -1
	add		a, a			; each of the seven default streams has
							; two bytes of offset data
							; the total number of bytes occupied,
							; held in a, forms the offset for the
							; new stream
	ld		b, 0			; set b to hold $00
	ld		c, a			; set the low byte of the offset
	add		hl, bc			; the offset is added to the base
							; address to give the correct location
							; in the streams table to store the
							; offset
	ld		(hl), e			; the low byte of the offset
	inc		hl				; move HL up one address
	ld		(hl), d			; the high byte of the offset
	im		1				; make sure it's off
	ret						;

org 40156
font:
	incbin	"font.bin"		; 96 characters centerd 6x8 font (768 bytes)

org 40924
tiles:
	incbin "tiles.bin"		; 256 tiles in ColorTILE format (16K)

org 57308
im_main:
	di
	push	af
	push	bc
	push	de
	push	hl
	ex		af,af'
	exx
	push	af
	push	bc
	push	de
	push	hl
	ld		(sp_cache), sp
	ld		bc, 491

delay_loop:
	nop
	nop
	dec		bc
	ld		a, b
	or		c
	jp		nz, delay_loop

	nop		; timing adjustments
	nop
	nop

atrs:
REPT 18, char_row
REPT 8, pixel_row
	ld		sp, 22579 + (32 * char_row)
	ld		bc, pixel_row * 256 + pixel_row		; column 01, 00
	ld		de, pixel_row * 256 + pixel_row		; column 03, 02
	ld		hl, pixel_row * 256 + pixel_row		; column 05, 04
	exx
	ld		de, pixel_row * 256 + pixel_row		; column 07, 06
	ld		hl, pixel_row * 256 + pixel_row		; column 09, 08
	ld		bc, pixel_row * 256 + pixel_row		; column 17, 16
	push	bc
	ld		bc, pixel_row * 256 + pixel_row		; column 15, 14
	push 	bc
	ld		bc, pixel_row * 256 + pixel_row		; column 13, 12
	push 	bc
	ld		bc, pixel_row * 256 + pixel_row		; column 11, 10
	push	bc
	push 	hl
	push 	de	
	exx
	push 	hl
	push 	de
	push 	bc
ENDM
ENDM

	ld		sp, (sp_cache)

set_jump:
	call	t00		

	pop		hl
	pop		de
	pop		bc
	pop		af
	exx
	ex		af,af'
	pop		hl
	pop		de
	pop		bc
	pop		af
	ei
	jp		56

x00 		equ atrs + 4
x02 		equ x00 + 3
x04 		equ x02 + 3
x06			equ x04 + 4
x08 		equ x06 + 3
x10 		equ x08 + 15
x12 		equ x10 - 4
x14 		equ x12 - 4
x16			equ x14 - 4

t00:
 	ld		hl, tile_map		; point to tile map
	call	get_tile_pos
	ld		de, $4021			; screen bitmap address
	call	output_bmp
	ld		de, $4041 			; next cell down
	call	output_bmp
	ld		de, x00
	call	output_atr

 	ld		hl, tile_map + 1	; point to tile map
	call	get_tile_pos
	ld		de, $4023			; screen bitmap address
	call	output_bmp
	ld		de, $4043 			; next cell down
	call	output_bmp
	ld		de, x02
	call	output_atr

 	ld		hl, tile_map + 2	; point to tile map
	call	get_tile_pos
	ld		de, $4025			; screen bitmap address
	call	output_bmp
	ld		de, $4045 			; next cell down
	call	output_bmp
	ld		de, x04
	call	output_atr

	ld		de, t01			; modify jump address
	ld		hl, set_jump + 1
	ld		(hl), e
	inc		hl
	ld		(hl), d
	ret

t01:
 	ld		hl, tile_map + 3	; point to tile map
	call	get_tile_pos
	ld		de, $4027			; screen bitmap address
	call	output_bmp
	ld		de, $4047 			; next cell down
	call	output_bmp
	ld		de, x06
	call	output_atr

 	ld		hl, tile_map + 4	; point to tile map
	call	get_tile_pos
	ld		de, $4029			; screen bitmap address
	call	output_bmp
	ld		de, $4049 			; next cell down
	call	output_bmp
	ld		de, x08
	call	output_atr

 	ld		hl, tile_map + 5	; point to tile map
	call	get_tile_pos
	ld		de, $402b			; screen bitmap address
	call	output_bmp
	ld		de, $404b 			; next cell down
	call	output_bmp
	ld		de, x10
	call	output_atr

	ld		de, t02			; modify jump address
	ld		hl, set_jump + 1
	ld		(hl), e
	inc		hl
	ld		(hl), d
	ret

t02:
 	ld		hl, tile_map + 6	; point to tile map
	call	get_tile_pos
	ld		de, $402d			; screen bitmap address
	call	output_bmp
	ld		de, $404d 			; next cell down
	call	output_bmp
	ld		de, x12
	call	output_atr

 	ld		hl, tile_map + 7	; point to tile map
	call	get_tile_pos
	ld		de, $402f			; screen bitmap address
	call	output_bmp
	ld		de, $404f 			; next cell down
	call	output_bmp
	ld		de, x14
	call	output_atr

 	ld		hl, tile_map + 8	; point to tile map
	call	get_tile_pos
	ld		de, $4031			; screen bitmap address
	call	output_bmp
	ld		de, $4051 			; next cell down
	call	output_bmp
	ld		de, x16
	call	output_atr

	ld		de, t03			; modify jump address
	ld		hl, set_jump + 1
	ld		(hl), e
	inc		hl
	ld		(hl), d
	ret

t03:
 	ld		hl, tile_map + 9	; point to tile map
	call	get_tile_pos
	ld		de, $4061			; screen bitmap address
	call	output_bmp
	ld		de, $4081 			; next cell down
	call	output_bmp
	ld		de, x00 + 656
	call	output_atr

 	ld		hl, tile_map + 10	; point to tile map
	call	get_tile_pos
	ld		de, $4063			; screen bitmap address
	call	output_bmp
	ld		de, $4083 			; next cell down
	call	output_bmp
	ld		de, x02 + 656
	call	output_atr

 	ld		hl, tile_map + 11	; point to tile map
	call	get_tile_pos
	ld		de, $4065			; screen bitmap address
	call	output_bmp
	ld		de, $4085 			; next cell down
	call	output_bmp
	ld		de, x04 + 656
	call	output_atr

	ld		de, t04			; modify jump address
	ld		hl, set_jump + 1
	ld		(hl), e
	inc		hl
	ld		(hl), d
	ret

t04:
 	ld		hl, tile_map + 12	; point to tile map
	call	get_tile_pos
	ld		de, $4067			; screen bitmap address
	call	output_bmp
	ld		de, $4087 			; next cell down
	call	output_bmp
	ld		de, x06 + 656
	call	output_atr

 	ld		hl, tile_map + 13	; point to tile map
	call	get_tile_pos
	ld		de, $4069			; screen bitmap address
	call	output_bmp
	ld		de, $4089 			; next cell down
	call	output_bmp
	ld		de, x08 + 656
	call	output_atr

 	ld		hl, tile_map + 14	; point to tile map
	call	get_tile_pos
	ld		de, $406b			; screen bitmap address
	call	output_bmp
	ld		de, $408b 			; next cell down
	call	output_bmp
	ld		de, x10 + 656
	call	output_atr

	ld		de, t05			; modify jump address
	ld		hl, set_jump + 1
	ld		(hl), e
	inc		hl
	ld		(hl), d
	ret

t05:
 	ld		hl, tile_map + 15	; point to tile map
	call	get_tile_pos
	ld		de, $406d			; screen bitmap address
	call	output_bmp
	ld		de, $408d 			; next cell down
	call	output_bmp
	ld		de, x12 + 656
	call	output_atr

 	ld		hl, tile_map + 16	; point to tile map
	call	get_tile_pos
	ld		de, $406f			; screen bitmap address
	call	output_bmp
	ld		de, $408f 			; next cell down
	call	output_bmp
	ld		de, x14 + 656
	call	output_atr

 	ld		hl, tile_map + 17	; point to tile map
	call	get_tile_pos
	ld		de, $4071			; screen bitmap address
	call	output_bmp
	ld		de, $4091 			; next cell down
	call	output_bmp
	ld		de, x16 + 656
	call	output_atr

	ld		de, t06			; modify jump address
	ld		hl, set_jump + 1
	ld		(hl), e
	inc		hl
	ld		(hl), d
	ret

t06:
 	ld		hl, tile_map + 18	; point to tile map
	call	get_tile_pos
	ld		de, $40a1			; screen bitmap address
	call	output_bmp
	ld		de, $40c1 			; next cell down
	call	output_bmp
	ld		de, x00 + 1312
	call	output_atr

 	ld		hl, tile_map + 19	; point to tile map
	call	get_tile_pos
	ld		de, $40a3			; screen bitmap address
	call	output_bmp
	ld		de, $40c3 			; next cell down
	call	output_bmp
	ld		de, x02 + 1312
	call	output_atr

 	ld		hl, tile_map + 20	; point to tile map
	call	get_tile_pos
	ld		de, $40a5			; screen bitmap address
	call	output_bmp
	ld		de, $40c5 			; next cell down
	call	output_bmp
	ld		de, x04 + 1312
	call	output_atr

	ld		de, t07			; modify jump address
	ld		hl, set_jump + 1
	ld		(hl), e
	inc		hl
	ld		(hl), d
	ret

t07:
 	ld		hl, tile_map + 21	; point to tile map
	call	get_tile_pos
	ld		de, $40a7			; screen bitmap address
	call	output_bmp
	ld		de, $40c7 			; next cell down
	call	output_bmp
	ld		de, x06 + 1312
	call	output_atr

 	ld		hl, tile_map + 22	; point to tile map
	call	get_tile_pos
	ld		de, $40a9			; screen bitmap address
	call	output_bmp
	ld		de, $40c9 			; next cell down
	call	output_bmp
	ld		de, x08 + 1312
	call	output_atr

 	ld		hl, tile_map + 23	; point to tile map
	call	get_tile_pos
	ld		de, $40ab			; screen bitmap address
	call	output_bmp
	ld		de, $40cb 			; next cell down
	call	output_bmp
	ld		de, x10 + 1312
	call	output_atr

	ld		de, t08			; modify jump address
	ld		hl, set_jump + 1
	ld		(hl), e
	inc		hl
	ld		(hl), d
	ret

t08:
 	ld		hl, tile_map + 24	; point to tile map
	call	get_tile_pos
	ld		de, $40ad			; screen bitmap address
	call	output_bmp
	ld		de, $40cd 			; next cell down
	call	output_bmp
	ld		de, x12 + 1312
	call	output_atr

 	ld		hl, tile_map + 25	; point to tile map
	call	get_tile_pos
	ld		de, $40af			; screen bitmap address
	call	output_bmp
	ld		de, $40cf 			; next cell down
	call	output_bmp
	ld		de, x14 + 1312
	call	output_atr

 	ld		hl, tile_map + 26	; point to tile map
	call	get_tile_pos
	ld		de, $40b1			; screen bitmap address
	call	output_bmp
	ld		de, $40d1 			; next cell down
	call	output_bmp
	ld		de, x16 + 1312
	call	output_atr

	ld		de, t09			; modify jump address
	ld		hl, set_jump + 1
	ld		(hl), e
	inc		hl
	ld		(hl), d
	ret

t09:
 	ld		hl, tile_map + 27	; point to tile map
	call	get_tile_pos
	ld		de, $40e1			; screen bitmap address
	call	output_bmp
	ld		de, $4801 			; next cell down
	call	output_bmp
	ld		de, x00 + 1968
	call	output_atr

 	ld		hl, tile_map + 28	; point to tile map
	call	get_tile_pos
	ld		de, $40e3			; screen bitmap address
	call	output_bmp
	ld		de, $4803 			; next cell down
	call	output_bmp
	ld		de, x02 + 1968
	call	output_atr

 	ld		hl, tile_map + 29	; point to tile map
	call	get_tile_pos
	ld		de, $40e5			; screen bitmap address
	call	output_bmp
	ld		de, $4805 			; next cell down
	call	output_bmp
	ld		de, x04 + 1968
	call	output_atr

	ld		de, t10			; modify jump address
	ld		hl, set_jump + 1
	ld		(hl), e
	inc		hl
	ld		(hl), d
	ret

t10:
 	ld		hl, tile_map + 30	; point to tile map
	call	get_tile_pos
	ld		de, $40e7			; screen bitmap address
	call	output_bmp
	ld		de, $4807 			; next cell down
	call	output_bmp
	ld		de, x06 + 1968
	call	output_atr

 	ld		hl, tile_map + 31	; point to tile map
	call	get_tile_pos
	ld		de, $40e9			; screen bitmap address
	call	output_bmp
	ld		de, $4809 			; next cell down
	call	output_bmp
	ld		de, x08 + 1968
	call	output_atr

 	ld		hl, tile_map + 32	; point to tile map
	call	get_tile_pos
	ld		de, $40eb			; screen bitmap address
	call	output_bmp
	ld		de, $480b 			; next cell down
	call	output_bmp
	ld		de, x10 + 1968
	call	output_atr

	ld		de, t11			; modify jump address
	ld		hl, set_jump + 1
	ld		(hl), e
	inc		hl
	ld		(hl), d
	ret

t11:
 	ld		hl, tile_map + 33	; point to tile map
	call	get_tile_pos
	ld		de, $40ed			; screen bitmap address
	call	output_bmp
	ld		de, $480d 			; next cell down
	call	output_bmp
	ld		de, x12 + 1968
	call	output_atr

 	ld		hl, tile_map + 34	; point to tile map
	call	get_tile_pos
	ld		de, $40ef			; screen bitmap address
	call	output_bmp
	ld		de, $480f 			; next cell down
	call	output_bmp
	ld		de, x14 + 1968
	call	output_atr

 	ld		hl, tile_map + 35	; point to tile map
	call	get_tile_pos
	ld		de, $40f1			; screen bitmap address
	call	output_bmp
	ld		de, $4811 			; next cell down
	call	output_bmp
	ld		de, x16 + 1968
	call	output_atr

	ld		de, t12			; modify jump address
	ld		hl, set_jump + 1
	ld		(hl), e
	inc		hl
	ld		(hl), d
	ret

t12:
 	ld		hl, tile_map + 36	; point to tile map
	call	get_tile_pos
	ld		de, $4821			; screen bitmap address
	call	output_bmp
	ld		de, $4841 			; next cell down
	call	output_bmp
	ld		de, x00 + 2624
	call	output_atr

 	ld		hl, tile_map + 37	; point to tile map
	call	get_tile_pos
	ld		de, $4823			; screen bitmap address
	call	output_bmp
	ld		de, $4843 			; next cell down
	call	output_bmp
	ld		de, x02 + 2624
	call	output_atr

 	ld		hl, tile_map + 38	; point to tile map
	call	get_tile_pos
	ld		de, $4825			; screen bitmap address
	call	output_bmp
	ld		de, $4845 			; next cell down
	call	output_bmp
	ld		de, x04 + 2624
	call	output_atr

	ld		de, t13				; modify jump address
	ld		hl, set_jump + 1
	ld		(hl), e
	inc		hl
	ld		(hl), d
	ret

t13:
 	ld		hl, tile_map + 39	; point to tile map
	call	get_tile_pos
	ld		de, $4827			; screen bitmap address
	call	output_bmp
	ld		de, $4847 			; next cell down
	call	output_bmp
	ld		de, x06 + 2624
	call	output_atr

 	ld		hl, tile_map + 40	; point to tile map
	call	get_tile_pos
	ld		de, $4829			; screen bitmap address
	call	output_bmp
	ld		de, $4849 			; next cell down
	call	output_bmp
	ld		de, x08 + 2624
	call	output_atr

 	ld		hl, tile_map + 41	; point to tile map
	call	get_tile_pos
	ld		de, $482b			; screen bitmap address
	call	output_bmp
	ld		de, $484b 			; next cell down
	call	output_bmp
	ld		de, x10 + 2624
	call	output_atr

	ld		de, t14			; modify jump address
	ld		hl, set_jump + 1
	ld		(hl), e
	inc		hl
	ld		(hl), d
	ret

t14:
 	ld		hl, tile_map + 42	; point to tile map
	call	get_tile_pos
	ld		de, $482d			; screen bitmap address
	call	output_bmp
	ld		de, $484d 			; next cell down
	call	output_bmp
	ld		de, x12 + 2624
	call	output_atr

 	ld		hl, tile_map + 43	; point to tile map
	call	get_tile_pos
	ld		de, $482f			; screen bitmap address
	call	output_bmp
	ld		de, $484f 			; next cell down
	call	output_bmp
	ld		de, x14 + 2624
	call	output_atr

 	ld		hl, tile_map + 44	; point to tile map
	call	get_tile_pos
	ld		de, $4831			; screen bitmap address
	call	output_bmp
	ld		de, $4851 			; next cell down
	call	output_bmp
	ld		de, x16 + 2624
	call	output_atr

	ld		de, t15			; modify jump address
	ld		hl, set_jump + 1
	ld		(hl), e
	inc		hl
	ld		(hl), d
	ret

t15:
 	ld		hl, tile_map + 45	; point to tile map
	call	get_tile_pos
	ld		de, $4861			; screen bitmap address
	call	output_bmp
	ld		de, $4881 			; next cell down
	call	output_bmp
	ld		de, x00 + 3280
	call	output_atr

 	ld		hl, tile_map + 46	; point to tile map
	call	get_tile_pos
	ld		de, $4863			; screen bitmap address
	call	output_bmp
	ld		de, $4883 			; next cell down
	call	output_bmp
	ld		de, x02 + 3280
	call	output_atr

 	ld		hl, tile_map + 47	; point to tile map
	call	get_tile_pos
	ld		de, $4865			; screen bitmap address
	call	output_bmp
	ld		de, $4885 			; next cell down
	call	output_bmp
	ld		de, x04 + 3280
	call	output_atr

	ld		de, t16				; modify jump address
	ld		hl, set_jump + 1
	ld		(hl), e
	inc		hl
	ld		(hl), d
	ret

t16:
 	ld		hl, tile_map + 48	; point to tile map
	call	get_tile_pos
	ld		de, $4867			; screen bitmap address
	call	output_bmp
	ld		de, $4887 			; next cell down
	call	output_bmp
	ld		de, x06 + 3280
	call	output_atr

 	ld		hl, tile_map + 49	; point to tile map
	call	get_tile_pos
	ld		de, $4869			; screen bitmap address
	call	output_bmp
	ld		de, $4889 			; next cell down
	call	output_bmp
	ld		de, x08 + 3280
	call	output_atr

 	ld		hl, tile_map + 50	; point to tile map
	call	get_tile_pos
	ld		de, $486b			; screen bitmap address
	call	output_bmp
	ld		de, $488b 			; next cell down
	call	output_bmp
	ld		de, x10 + 3280
	call	output_atr

	ld		de, t17			; modify jump address
	ld		hl, set_jump + 1
	ld		(hl), e
	inc		hl
	ld		(hl), d
	ret

t17:
 	ld		hl, tile_map + 51	; point to tile map
	call	get_tile_pos
	ld		de, $486d			; screen bitmap address
	call	output_bmp
	ld		de, $488d 			; next cell down
	call	output_bmp
	ld		de, x12 + 3280
	call	output_atr

 	ld		hl, tile_map + 52	; point to tile map
	call	get_tile_pos
	ld		de, $486f			; screen bitmap address
	call	output_bmp
	ld		de, $488f 			; next cell down
	call	output_bmp
	ld		de, x14 + 3280
	call	output_atr

 	ld		hl, tile_map + 53	; point to tile map
	call	get_tile_pos
	ld		de, $4871			; screen bitmap address
	call	output_bmp
	ld		de, $4891 			; next cell down
	call	output_bmp
	ld		de, x16 + 3280
	call	output_atr

	ld		de, t18			; modify jump address
	ld		hl, set_jump + 1
	ld		(hl), e
	inc		hl
	ld		(hl), d
	ret

t18:
 	ld		hl, tile_map + 54	; point to tile map
	call	get_tile_pos
	ld		de, $48a1			; screen bitmap address
	call	output_bmp
	ld		de, $48c1 			; next cell down
	call	output_bmp
	ld		de, x00 + 3936
	call	output_atr

 	ld		hl, tile_map + 55	; point to tile map
	call	get_tile_pos
	ld		de, $48a3			; screen bitmap address
	call	output_bmp
	ld		de, $48c3 			; next cell down
	call	output_bmp
	ld		de, x02 + 3936
	call	output_atr

 	ld		hl, tile_map + 56	; point to tile map
	call	get_tile_pos
	ld		de, $48a5			; screen bitmap address
	call	output_bmp
	ld		de, $48c5 			; next cell down
	call	output_bmp
	ld		de, x04 + 3936
	call	output_atr

	ld		de, t19				; modify jump address
	ld		hl, set_jump + 1
	ld		(hl), e
	inc		hl
	ld		(hl), d
	ret

t19:
 	ld		hl, tile_map + 57	; point to tile map
	call	get_tile_pos
	ld		de, $48a7			; screen bitmap address
	call	output_bmp
	ld		de, $48c7 			; next cell down
	call	output_bmp
	ld		de, x06 + 3936
	call	output_atr

 	ld		hl, tile_map + 58	; point to tile map
	call	get_tile_pos
	ld		de, $48a9			; screen bitmap address
	call	output_bmp
	ld		de, $48c9 			; next cell down
	call	output_bmp
	ld		de, x08 + 3936
	call	output_atr

 	ld		hl, tile_map + 59	; point to tile map
	call	get_tile_pos
	ld		de, $48ab			; screen bitmap address
	call	output_bmp
	ld		de, $48cb 			; next cell down
	call	output_bmp
	ld		de, x10 + 3936
	call	output_atr

	ld		de, t20			; modify jump address
	ld		hl, set_jump + 1
	ld		(hl), e
	inc		hl
	ld		(hl), d
	ret

t20:
 	ld		hl, tile_map + 60	; point to tile map
	call	get_tile_pos
	ld		de, $48ad			; screen bitmap address
	call	output_bmp
	ld		de, $48cd 			; next cell down
	call	output_bmp
	ld		de, x12 + 3936
	call	output_atr

 	ld		hl, tile_map + 61	; point to tile map
	call	get_tile_pos
	ld		de, $48af			; screen bitmap address
	call	output_bmp
	ld		de, $48cf 			; next cell down
	call	output_bmp
	ld		de, x14 + 3936
	call	output_atr

 	ld		hl, tile_map + 62	; point to tile map
	call	get_tile_pos
	ld		de, $48b1			; screen bitmap address
	call	output_bmp
	ld		de, $48d1 			; next cell down
	call	output_bmp
	ld		de, x16 + 3936
	call	output_atr

	ld		de, t21			; modify jump address
	ld		hl, set_jump + 1
	ld		(hl), e
	inc		hl
	ld		(hl), d
	ret

t21:
 	ld		hl, tile_map + 63	; point to tile map
	call	get_tile_pos
	ld		de, $48e1			; screen bitmap address
	call	output_bmp
	ld		de, $5001 			; next cell down
	call	output_bmp
	ld		de, x00 + 4592
	call	output_atr

 	ld		hl, tile_map + 64	; point to tile map
	call	get_tile_pos
	ld		de, $48e3			; screen bitmap address
	call	output_bmp
	ld		de, $5003 			; next cell down
	call	output_bmp
	ld		de, x02 + 4592
	call	output_atr

 	ld		hl, tile_map + 65	; point to tile map
	call	get_tile_pos
	ld		de, $48e5			; screen bitmap address
	call	output_bmp
	ld		de, $5005 			; next cell down
	call	output_bmp
	ld		de, x04 + 4592
	call	output_atr

	ld		de, t22				; modify jump address
	ld		hl, set_jump + 1
	ld		(hl), e
	inc		hl
	ld		(hl), d
	ret

t22:
 	ld		hl, tile_map + 66	; point to tile map
	call	get_tile_pos
	ld		de, $48e7			; screen bitmap address
	call	output_bmp
	ld		de, $5007 			; next cell down
	call	output_bmp
	ld		de, x06 + 4592
	call	output_atr

 	ld		hl, tile_map + 67	; point to tile map
	call	get_tile_pos
	ld		de, $48e9			; screen bitmap address
	call	output_bmp
	ld		de, $5009 			; next cell down
	call	output_bmp
	ld		de, x08 + 4592
	call	output_atr

 	ld		hl, tile_map + 68	; point to tile map
	call	get_tile_pos
	ld		de, $48eb			; screen bitmap address
	call	output_bmp
	ld		de, $500b 			; next cell down
	call	output_bmp
	ld		de, x10 + 4592
	call	output_atr

	ld		de, t23			; modify jump address
	ld		hl, set_jump + 1
	ld		(hl), e
	inc		hl
	ld		(hl), d
	ret

t23:
 	ld		hl, tile_map + 69	; point to tile map
	call	get_tile_pos
	ld		de, $48ed			; screen bitmap address
	call	output_bmp
	ld		de, $500d 			; next cell down
	call	output_bmp
	ld		de, x12 + 4592
	call	output_atr

 	ld		hl, tile_map + 70	; point to tile map
	call	get_tile_pos
	ld		de, $48ef			; screen bitmap address
	call	output_bmp
	ld		de, $500f 			; next cell down
	call	output_bmp
	ld		de, x14 + 4592
	call	output_atr

 	ld		hl, tile_map + 71	; point to tile map
	call	get_tile_pos
	ld		de, $48f1			; screen bitmap address
	call	output_bmp
	ld		de, $5011 			; next cell down
	call	output_bmp
	ld		de, x16 + 4592
	call	output_atr

	ld		de, t24			; modify jump address
	ld		hl, set_jump + 1
	ld		(hl), e
	inc		hl
	ld		(hl), d
	ret

t24:
 	ld		hl, tile_map + 72	; point to tile map
	call	get_tile_pos
	ld		de, $5021			; screen bitmap address
	call	output_bmp
	ld		de, $5041 			; next cell down
	call	output_bmp
	ld		de, x00 + 5248
	call	output_atr

 	ld		hl, tile_map + 73	; point to tile map
	call	get_tile_pos
	ld		de, $5023			; screen bitmap address
	call	output_bmp
	ld		de, $5043 			; next cell down
	call	output_bmp
	ld		de, x02 + 5248
	call	output_atr

 	ld		hl, tile_map + 74	; point to tile map
	call	get_tile_pos
	ld		de, $5025			; screen bitmap address
	call	output_bmp
	ld		de, $5045 			; next cell down
	call	output_bmp
	ld		de, x04 + 5248
	call	output_atr

	ld		de, t25				; modify jump address
	ld		hl, set_jump + 1
	ld		(hl), e
	inc		hl
	ld		(hl), d
	ret

t25:
 	ld		hl, tile_map + 75	; point to tile map
	call	get_tile_pos
	ld		de, $5027			; screen bitmap address
	call	output_bmp
	ld		de, $5047 			; next cell down
	call	output_bmp
	ld		de, x06 + 5248
	call	output_atr

 	ld		hl, tile_map + 76	; point to tile map
	call	get_tile_pos
	ld		de, $5029			; screen bitmap address
	call	output_bmp
	ld		de, $5049 			; next cell down
	call	output_bmp
	ld		de, x08 + 5248
	call	output_atr

 	ld		hl, tile_map + 77	; point to tile map
	call	get_tile_pos
	ld		de, $502b			; screen bitmap address
	call	output_bmp
	ld		de, $504b 			; next cell down
	call	output_bmp
	ld		de, x10 + 5248
	call	output_atr

	ld		de, t26			; modify jump address
	ld		hl, set_jump + 1
	ld		(hl), e
	inc		hl
	ld		(hl), d
	ret

t26:
 	ld		hl, tile_map + 78	; point to tile map
	call	get_tile_pos
	ld		de, $502d			; screen bitmap address
	call	output_bmp
	ld		de, $504d 			; next cell down
	call	output_bmp
	ld		de, x12 + 5248
	call	output_atr

 	ld		hl, tile_map + 79	; point to tile map
	call	get_tile_pos
	ld		de, $502f			; screen bitmap address
	call	output_bmp
	ld		de, $504f 			; next cell down
	call	output_bmp
	ld		de, x14 + 5248
	call	output_atr

 	ld		hl, tile_map + 80	; point to tile map
	call	get_tile_pos
	ld		de, $5031			; screen bitmap address
	call	output_bmp
	ld		de, $5051 			; next cell down
	call	output_bmp
	ld		de, x16 + 5248
	call	output_atr

	ld		de, t00			; modify jump address
	ld		hl, set_jump + 1
	ld		(hl), e
	inc		hl
	ld		(hl), d
	ret

get_tile_pos:
	ld		a, (hl)				; get value of first tile
	ld		h, 0
	ld		l, a
	add		hl, hl				; *2
	add		hl, hl				; *4
	add		hl, hl				; *8
	add		hl, hl				; *16
	add		hl, hl				; *32
	add		hl, hl				; *64
	ld		de, tiles			; address of tiles
	add		hl, de				; offset to definition
	ret

output_bmp:
	ld		b, 8
bitmap:
	ld		a, (hl)				; write first byte
	ld		(de), a
	inc		hl					; byte 2
	inc		de
	ld		a, (hl)				; write second byte
	ld		(de), a
	inc		hl					; to next position	
	dec		de
	inc		d
	djnz	bitmap
	ret

output_atr:	
	ld		b, 16
attribs:
	ld		a, (hl)
	ld		(de), a
	inc		hl
	inc		de
	ld		a, (hl)
	ld		(de), a
	inc		hl
	push	hl
	ld		hl, 40
	add		hl, de
	ex		de, hl
	pop		hl
	djnz	attribs	
	ret

org 65524
vector:
	jp		im_main

; variables
sp_cache:
	defw	0
chrad:
	defw	0
atflg:
	defb	0

org 65535
	defb	24					; combined with first ROM byte forms JR vector
